/** fichier libcom.c **/

/*****************************************************************/
/** Ce fichier contient des fonctions concernant les sockets.   **/
/*****************************************************************/

/**** Fichiers d'inclusion ****/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <netinet/tcp.h>
#include <netdb.h>

#include <arpa/nameser.h>
#include <resolv.h>
#include <poll.h>
#include <arpa/inet.h>

#include "libcom.h"
//#include "../SMTP/lib_SMTP.h"


/**** Constantes ****/
#define MAX_PRIORITY 16
#define MAX_LIGNE 250


/**** Variables globales *****/

/**** Fonctions de gestion des sockets ****/

int initialisationServeur(char *service,int connexions)
{
	struct addrinfo precisions,*resultat,*origine;
	int statut;
	int s;

	/* Construction de la structure adresse */
	memset(&precisions,0,sizeof precisions);
	precisions.ai_family=AF_UNSPEC;
	precisions.ai_socktype=SOCK_STREAM;
	precisions.ai_flags=AI_PASSIVE;
	statut=getaddrinfo(NULL,service,&precisions,&origine);
	if(statut<0){ perror("initialisationServeur.getaddrinfo"); exit(EXIT_FAILURE); }

	struct addrinfo *p;
	for(p=origine,resultat=origine;p!=NULL;p=p->ai_next)
  		if(p->ai_family==AF_INET6){ resultat=p; break; }

	/* Creation d'une socket */
	s=socket(resultat->ai_family,resultat->ai_socktype,resultat->ai_protocol);
	if(s<0){ perror("initialisationServeur.socket"); exit(EXIT_FAILURE); }

	/* Options utiles */
	int vrai=1;
	if(setsockopt(s,SOL_SOCKET,SO_REUSEADDR,&vrai,sizeof(vrai))<0)
	{
  		perror("initialisationServeur.setsockopt (REUSEADDR)");
 		exit(EXIT_FAILURE);
 	}

	if(setsockopt(s,IPPROTO_TCP,TCP_NODELAY,&vrai,sizeof(vrai))<0)
	{
  		perror("initialisationServeur.setsockopt (NODELAY)");
  		exit(EXIT_FAILURE);
  	}

	/* Specification de l'adresse de la socket */
	statut=bind(s,resultat->ai_addr,resultat->ai_addrlen);

	if(statut<0) return -1;

	/* Liberation de la structure d'informations */
	freeaddrinfo(origine);

	/* Taille de la queue d'attente */
	statut=listen(s,connexions);
	if(statut<0) return -1;
	return s;
}




int boucleServeur(int ecoute,int (*traitement)(int))
{
	int dialogue;
	while(1)
	{
    	/* Attente d'une connexion */
		if((dialogue=accept(ecoute,NULL,NULL))<0) return -1;
    
		/* Passage de la socket de dialogue a la fonction de traitement */
		if(traitement(dialogue)<0){ shutdown(ecoute,SHUT_RDWR); return 0;}
    }
}



int nomVersAdresse(char *hote,struct sockaddr_storage *padresse)
{
/* On regarde deja s'il s'agit d'une adresse IP */

struct sockaddr_in *ipv4_sock=(struct sockaddr_in *)padresse;
unsigned char *ipv4=(unsigned char *)&ipv4_sock->sin_addr;
if(padresse->ss_family!=AF_INET6){
  int size=inet_pton(AF_INET,hote,ipv4);
  if(size>0){
    padresse->ss_family=AF_INET;
    return 0;
    }
  }
struct sockaddr_in6 *ipv6_sock=(struct sockaddr_in6 *)padresse;
unsigned char *ipv6=(unsigned char *)&ipv6_sock->sin6_addr;
if(padresse->ss_family!=AF_INET){
  int size=inet_pton(AF_INET6,hote,ipv6);
  if(size>0){
    padresse->ss_family=AF_INET6;
    return 0;
    }
  }

/* Sinon on cherche l'adresse via le DNS */
if(padresse->ss_family!=AF_INET){
  struct hostent *h=gethostbyname2(hote,AF_INET6);
  if(h!=NULL){
    memcpy(ipv6,h->h_addr_list[0],h->h_length);
    padresse->ss_family=AF_INET6;
    return 0;
    }
  }
if(padresse->ss_family!=AF_INET6){
  struct hostent *h=gethostbyname2(hote,AF_INET);
  if(h!=NULL){
    memcpy(ipv4,h->h_addr_list[0],h->h_length);
    padresse->ss_family=AF_INET;
    return 0;
    }
  }
return -1;
}






int connexionServeur(char *hote,int port)
{
	int s;
	struct sockaddr_in adresse;
	socklen_t taille=sizeof adresse;

	/* Creation d'une socket */
	s=socket(PF_INET,SOCK_STREAM,0);
	if(s<0){
	 perror("connexionServeur.socket");
	 exit(-1);
	 }
	 
	 /* Connection de la socket a l'hote */
	adresse.sin_family=AF_INET;
	if(nomVersAdresse(hote,(void *)&adresse)<0) return -1;
	adresse.sin_port=htons(port);
	if(connect(s,(struct sockaddr *)&adresse,taille)<0) return -1;
	
	return 0;
}
 